package com.sun.java.swing.plaf.windows;

import java.awt.image.IndexColorModel;
import java.awt.image.BufferedImage;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import javax.swing.ButtonModel;
import javax.swing.plaf.basic.BasicArrowButton;
import java.awt.Insets;
import javax.swing.JScrollBar;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import javax.swing.JButton;
import java.awt.Color;
import javax.swing.UIManager;
import java.awt.Component;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Dimension;
import javax.swing.plaf.basic.BasicScrollBarUI;

public class WindowsScrollBarUI extends BasicScrollBarUI
{
    private WindowsScrollBarUI.Grid thumbGrid;
    private WindowsScrollBarUI.Grid highlightGrid;
    private Dimension horizontalThumbSize;
    private Dimension verticalThumbSize;
    
    public WindowsScrollBarUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new WindowsScrollBarUI();
    }
    
    @Override
    protected void installDefaults() {
        super.installDefaults();
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            this.scrollbar.setBorder(null);
            this.horizontalThumbSize = getSize(this.scrollbar, xp, TMSchema.Part.SBP_THUMBBTNHORZ);
            this.verticalThumbSize = getSize(this.scrollbar, xp, TMSchema.Part.SBP_THUMBBTNVERT);
        }
        else {
            this.horizontalThumbSize = null;
            this.verticalThumbSize = null;
        }
    }
    
    private static Dimension getSize(Component component, XPStyle xpStyle, TMSchema.Part part) {
        XPStyle.Skin skin = xpStyle.getSkin(component, part);
        return new Dimension(skin.getWidth(), skin.getHeight());
    }
    
    @Override
    protected Dimension getMinimumThumbSize() {
        if (this.horizontalThumbSize == null || this.verticalThumbSize == null) {
            return super.getMinimumThumbSize();
        }
        return (0 == this.scrollbar.getOrientation()) ? this.horizontalThumbSize : this.verticalThumbSize;
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        WindowsScrollBarUI.Grid grid = null;
        this.highlightGrid = grid;
        this.thumbGrid = grid;
    }
    
    @Override
    protected void configureScrollBarColors() {
        super.configureScrollBarColors();
        Color color = UIManager.getColor("ScrollBar.trackForeground");
        if (color != null && this.trackColor != null) {
            this.thumbGrid = WindowsScrollBarUI.Grid.getGrid(color, this.trackColor);
        }
        Color color2 = UIManager.getColor("ScrollBar.trackHighlightForeground");
        if (color2 != null && this.trackHighlightColor != null) {
            this.highlightGrid = WindowsScrollBarUI.Grid.getGrid(color2, this.trackHighlightColor);
        }
    }
    
    @Override
    protected JButton createDecreaseButton(int n) {
        return new WindowsScrollBarUI.WindowsArrowButton(n, UIManager.getColor("ScrollBar.thumb"), UIManager.getColor("ScrollBar.thumbShadow"), UIManager.getColor("ScrollBar.thumbDarkShadow"), UIManager.getColor("ScrollBar.thumbHighlight"));
    }
    
    @Override
    protected JButton createIncreaseButton(int n) {
        return new WindowsScrollBarUI.WindowsArrowButton(n, UIManager.getColor("ScrollBar.thumb"), UIManager.getColor("ScrollBar.thumbShadow"), UIManager.getColor("ScrollBar.thumbDarkShadow"), UIManager.getColor("ScrollBar.thumbHighlight"));
    }
    
    @Override
    protected BasicScrollBarUI.ArrowButtonListener createArrowButtonListener() {
        if (XPStyle.isVista()) {
            return new BasicScrollBarUI.ArrowButtonListener() {
                @Override
                public void mouseEntered(MouseEvent e) {
                    this.repaint();
                    super.mouseEntered(e);
                }
                
                @Override
                public void mouseExited(MouseEvent e) {
                    this.repaint();
                    super.mouseExited(e);
                }
                
                private void repaint() {
                    WindowsScrollBarUI.this.scrollbar.repaint();
                }
            };
        }
        return super.createArrowButtonListener();
    }
    
    @Override
    protected void paintTrack(Graphics g, JComponent c, Rectangle trackBounds) {
        int n = this.scrollbar.getOrientation() == 1;
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            JScrollBar scrollBar = (JScrollBar)c;
            TMSchema.State state = TMSchema.State.NORMAL;
            if (!scrollBar.isEnabled()) {
                state = TMSchema.State.DISABLED;
            }
            xp.getSkin(scrollBar, n ? TMSchema.Part.SBP_LOWERTRACKVERT : TMSchema.Part.SBP_LOWERTRACKHORZ).paintSkin(g, trackBounds, state);
        }
        else if (this.thumbGrid == null) {
            super.paintTrack(g, c, trackBounds);
        }
        else {
            this.thumbGrid.paint(g, trackBounds.x, trackBounds.y, trackBounds.width, trackBounds.height);
            if (this.trackHighlight == 1) {
                this.paintDecreaseHighlight(g);
            }
            else if (this.trackHighlight == 2) {
                this.paintIncreaseHighlight(g);
            }
        }
    }
    
    @Override
    protected void paintThumb(Graphics g, JComponent c, Rectangle thumbBounds) {
        int n = this.scrollbar.getOrientation() == 1;
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            JScrollBar scrollBar = (JScrollBar)c;
            TMSchema.State state = TMSchema.State.NORMAL;
            if (!scrollBar.isEnabled()) {
                state = TMSchema.State.DISABLED;
            }
            else if (this.isDragging) {
                state = TMSchema.State.PRESSED;
            }
            else if (this.isThumbRollover()) {
                state = TMSchema.State.HOT;
            }
            else if (XPStyle.isVista() && ((this.incrButton != null && this.incrButton.getModel().isRollover()) || (this.decrButton != null && this.decrButton.getModel().isRollover()))) {
                state = TMSchema.State.HOVER;
            }
            TMSchema.Part part = n ? TMSchema.Part.SBP_THUMBBTNVERT : TMSchema.Part.SBP_THUMBBTNHORZ;
            xp.getSkin(scrollBar, part).paintSkin(g, thumbBounds, state);
            XPStyle.Skin skin = xp.getSkin(scrollBar, n ? TMSchema.Part.SBP_GRIPPERVERT : TMSchema.Part.SBP_GRIPPERHORZ);
            Insets margin = xp.getMargin(c, part, null, TMSchema.Prop.CONTENTMARGINS);
            if (margin == null || (n && thumbBounds.height - margin.top - margin.bottom >= skin.getHeight()) || (n == 0 && thumbBounds.width - margin.left - margin.right >= skin.getWidth())) {
                skin.paintSkin(g, thumbBounds.x + (thumbBounds.width - skin.getWidth()) / 2, thumbBounds.y + (thumbBounds.height - skin.getHeight()) / 2, skin.getWidth(), skin.getHeight(), state);
            }
        }
        else {
            super.paintThumb(g, c, thumbBounds);
        }
    }
    
    @Override
    protected void paintDecreaseHighlight(Graphics g) {
        if (this.highlightGrid == null) {
            super.paintDecreaseHighlight(g);
        }
        else {
            Insets insets = this.scrollbar.getInsets();
            Rectangle thumbBounds = this.getThumbBounds();
            int left;
            int top;
            int n;
            int n2;
            if (this.scrollbar.getOrientation() == 1) {
                left = insets.left;
                top = this.decrButton.getY() + this.decrButton.getHeight();
                n = this.scrollbar.getWidth() - (insets.left + insets.right);
                n2 = thumbBounds.y - top;
            }
            else {
                left = this.decrButton.getX() + this.decrButton.getHeight();
                top = insets.top;
                n = thumbBounds.x - left;
                n2 = this.scrollbar.getHeight() - (insets.top + insets.bottom);
            }
            this.highlightGrid.paint(g, left, top, n, n2);
        }
    }
    
    @Override
    protected void paintIncreaseHighlight(Graphics g) {
        if (this.highlightGrid == null) {
            super.paintDecreaseHighlight(g);
        }
        else {
            Insets insets = this.scrollbar.getInsets();
            Rectangle thumbBounds = this.getThumbBounds();
            int left;
            int top;
            int n;
            int n2;
            if (this.scrollbar.getOrientation() == 1) {
                left = insets.left;
                top = thumbBounds.y + thumbBounds.height;
                n = this.scrollbar.getWidth() - (insets.left + insets.right);
                n2 = this.incrButton.getY() - top;
            }
            else {
                left = thumbBounds.x + thumbBounds.width;
                top = insets.top;
                n = this.incrButton.getX() - left;
                n2 = this.scrollbar.getHeight() - (insets.top + insets.bottom);
            }
            this.highlightGrid.paint(g, left, top, n, n2);
        }
    }
    
    @Override
    protected void setThumbRollover(boolean thumbRollover) {
        int thumbRollover2 = this.isThumbRollover();
        super.setThumbRollover(thumbRollover);
        if (XPStyle.isVista() && (thumbRollover ? 1 : 0) != thumbRollover2) {
            this.scrollbar.repaint();
        }
    }
    
    private class WindowsArrowButton extends BasicArrowButton
    {
        public WindowsArrowButton(int direction, Color background, Color shadow, Color darkShadow, Color highlight) {
            super(direction, background, shadow, darkShadow, highlight);
        }
        
        public WindowsArrowButton(int direction) {
            super(direction);
        }
        
        @Override
        public void paint(Graphics g) {
            XPStyle xp = XPStyle.getXP();
            if (xp != null) {
                ButtonModel model = this.getModel();
                XPStyle.Skin skin = xp.getSkin(this, TMSchema.Part.SBP_ARROWBTN);
                TMSchema.State state = null;
                int n = XPStyle.isVista() && (WindowsScrollBarUI.this.isThumbRollover() || (this == WindowsScrollBarUI.this.incrButton && WindowsScrollBarUI.this.decrButton.getModel().isRollover()) || (this == WindowsScrollBarUI.this.decrButton && WindowsScrollBarUI.this.incrButton.getModel().isRollover()));
                if (model.isArmed() && model.isPressed()) {
                    switch (this.direction) {
                        case 1: {
                            state = TMSchema.State.UPPRESSED;
                            break;
                        }
                        case 5: {
                            state = TMSchema.State.DOWNPRESSED;
                            break;
                        }
                        case 7: {
                            state = TMSchema.State.LEFTPRESSED;
                            break;
                        }
                        case 3: {
                            state = TMSchema.State.RIGHTPRESSED;
                            break;
                        }
                    }
                }
                else if (!model.isEnabled()) {
                    switch (this.direction) {
                        case 1: {
                            state = TMSchema.State.UPDISABLED;
                            break;
                        }
                        case 5: {
                            state = TMSchema.State.DOWNDISABLED;
                            break;
                        }
                        case 7: {
                            state = TMSchema.State.LEFTDISABLED;
                            break;
                        }
                        case 3: {
                            state = TMSchema.State.RIGHTDISABLED;
                            break;
                        }
                    }
                }
                else if (model.isRollover() || model.isPressed()) {
                    switch (this.direction) {
                        case 1: {
                            state = TMSchema.State.UPHOT;
                            break;
                        }
                        case 5: {
                            state = TMSchema.State.DOWNHOT;
                            break;
                        }
                        case 7: {
                            state = TMSchema.State.LEFTHOT;
                            break;
                        }
                        case 3: {
                            state = TMSchema.State.RIGHTHOT;
                            break;
                        }
                    }
                }
                else if (!(n == 0)) {
                    switch (this.direction) {
                        case 1: {
                            state = TMSchema.State.UPHOVER;
                            break;
                        }
                        case 5: {
                            state = TMSchema.State.DOWNHOVER;
                            break;
                        }
                        case 7: {
                            state = TMSchema.State.LEFTHOVER;
                            break;
                        }
                        case 3: {
                            state = TMSchema.State.RIGHTHOVER;
                            break;
                        }
                    }
                }
                else {
                    switch (this.direction) {
                        case 1: {
                            state = TMSchema.State.UPNORMAL;
                            break;
                        }
                        case 5: {
                            state = TMSchema.State.DOWNNORMAL;
                            break;
                        }
                        case 7: {
                            state = TMSchema.State.LEFTNORMAL;
                            break;
                        }
                        case 3: {
                            state = TMSchema.State.RIGHTNORMAL;
                            break;
                        }
                    }
                }
                skin.paintSkin(g, 0, 0, this.getWidth(), this.getHeight(), state);
            }
            else {
                super.paint(g);
            }
        }
        
        @Override
        public Dimension getPreferredSize() {
            int n = 16;
            if (WindowsScrollBarUI.this.scrollbar != null) {
                switch (WindowsScrollBarUI.this.scrollbar.getOrientation()) {
                    case 1: {
                        n = WindowsScrollBarUI.this.scrollbar.getWidth();
                        break;
                    }
                    case 0: {
                        n = WindowsScrollBarUI.this.scrollbar.getHeight();
                        break;
                    }
                }
                n = Math.max(n, 5);
            }
            return new Dimension(n, n);
        }
    }
    
    private static class Grid
    {
        private static final int BUFFER_SIZE = 64;
        private static HashMap<String, WeakReference<WindowsScrollBarUI.Grid>> map;
        private BufferedImage image;
        
        public static WindowsScrollBarUI.Grid getGrid(Color color, Color color2) {
            String string = color.getRGB() + " " + color2.getRGB();
            WeakReference<Object> weakReference = (WeakReference)WindowsScrollBarUI.Grid.map.get(string);
            WindowsScrollBarUI.Grid referent = (weakReference == null) ? null : ((WindowsScrollBarUI.Grid)weakReference.get());
            if (referent == null) {
                referent = new Grid(color, color2);
                WindowsScrollBarUI.Grid.map.put(string, new WeakReference(referent));
                return referent;
            }
            return referent;
        }
        
        public Grid(Color color, Color color2) {
            this.image = new BufferedImage(64, 64, 13, new IndexColorModel(8, 2, new int[] { color.getRGB(), color2.getRGB() }, 0, false, -1, 0));
            Graphics graphics = this.image.getGraphics();
            try {
                graphics.setClip(0, 0, 64, 64);
                this.paintGrid(graphics, color, color2);
            }
            finally {
                graphics.dispose();
            }
        }
        
        public void paint(Graphics graphics, int a, int a2, int n, int n2) {
            Rectangle clipBounds = graphics.getClipBounds();
            int i = Math.max(a, clipBounds.x);
            int j = Math.max(a2, clipBounds.y);
            int min = Math.min(clipBounds.x + clipBounds.width, a + n);
            int min2 = Math.min(clipBounds.y + clipBounds.height, a2 + n2);
            if (min <= i || min2 <= j) {
                return;
            }
            int n3 = (i - a) % 2;
            for (; i < min; i += 64) {
                int n4 = (j - a2) % 2;
                int min3 = Math.min(64 - n3, min - i);
                for (; j < min2; j += 64) {
                    int min4 = Math.min(64 - n4, min2 - j);
                    graphics.drawImage(this.image, i, j, i + min3, j + min4, n3, n4, n3 + min3, n4 + min4, null);
                    if (n4 != 0) {
                        j -= n4;
                        n4 = 0;
                    }
                }
                if (n3 != 0) {
                    i -= n3;
                    n3 = 0;
                }
            }
            return;
        }
        
        private void paintGrid(Graphics graphics, Color color, Color color2) {
            Rectangle clipBounds = graphics.getClipBounds();
            graphics.setColor(color2);
            graphics.fillRect(clipBounds.x, clipBounds.y, clipBounds.width, clipBounds.height);
            graphics.setColor(color);
            graphics.translate(clipBounds.x, clipBounds.y);
            int width;
            int height;
            int i;
            for (width = clipBounds.width, height = clipBounds.height, i = clipBounds.x % 2; i < width - height; i += 2) {
                graphics.drawLine(i, 0, i + height, height);
            }
            for (; i < width; i += 2) {
                graphics.drawLine(i, 0, width, width - i);
            }
            int j;
            for (j = ((clipBounds.x % 2 == 0) ? 2 : 1); j < height - width; j += 2) {
                graphics.drawLine(0, j, width, j + width);
            }
            for (; j < height; j += 2) {
                graphics.drawLine(0, j, height - j, height);
            }
            graphics.translate(-clipBounds.x, -clipBounds.y);
        }
        
        static {
            WindowsScrollBarUI.Grid.map = new HashMap();
        }
    }
}
